/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.archivierung;

import java.io.File;
import java.io.IOException;
import java.util.UUID;

import jakarta.activation.DataHandler;
import jakarta.activation.FileDataSource;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.client_attribute.OzgCloudAccessPermission;
import de.ozgcloud.apilib.client_attribute.OzgCloudBooleanClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.callcontext.CallContextUser;
import de.ozgcloud.archive.common.xta.XtaService;
import de.ozgcloud.archive.export.ExportService;
import de.ozgcloud.command.Command;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.xta.client.model.XtaFile;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
class ArchiveService {

	public static final String ATTRIBUTE_NAME_ANTRAG_ARCHIVING = "ARCHIVING";
	private final XtaService xtaService;
	private final ExportService exportService;
	@Qualifier(ArchiveManagerConfiguration.OZGCLOUD_CLIENT_ATTRIBUTE_SERVICE_NAME)
	private final OzgCloudClientAttributeService ozgCloudAttributesService;
	private final OzgCloudVorgangIdMapper vorgangIdMapper;

	public void setVorgangArchiving(String vorgangId) {
		ozgCloudAttributesService.setClientAttribute(buildBooleanClientAttribute(true), vorgangIdMapper.fromString(vorgangId));
	}

	OzgCloudBooleanClientAttribute buildBooleanClientAttribute(boolean isArchiving) {
		return OzgCloudBooleanClientAttribute.builder()
				.clientName(CallContextUser.ARCHIVE_MANAGER_CLIENT_NAME)
				.access(OzgCloudAccessPermission.READ_ONLY)
				.attributeName(ATTRIBUTE_NAME_ANTRAG_ARCHIVING)
				.value(isArchiving)
				.build();
	}

	public void archiveVorgang(Command command) {
		xtaService.sendXdomeaFile(buildXdomeaXtaFile(command.getVorgangId()));
	}

	XtaFile buildXdomeaXtaFile(String vorgangId) {
		var fileNameId = createFileNameId();
		return XtaFile.builder()
				.name(exportService.buildXdomeaFileName(fileNameId))
				.content(createFileContent(vorgangId, fileNameId))
				.contentType("application/zip")
				.build();
	}

	String createFileNameId() {
		return UUID.randomUUID().toString();
	}

	DataHandler createFileContent(String vorgangId, String fileNameId) {
		try {
			var tempFile = createTempFile(fileNameId);
			var fileDataSource = new FileDataSource(tempFile);
			exportService.writeXdomeaFileContent(vorgangId, fileNameId, fileDataSource.getOutputStream());
			return new DataHandler(fileDataSource);
		} catch (IOException e) {
			throw new TechnicalException("Error on creating file content for xDomea file!", e);
		}
	}

	File createTempFile(String fileNameId) throws IOException {
		var tempFile = File.createTempFile(fileNameId, ".zip");
		tempFile.deleteOnExit();
		return tempFile;
	}
}
