/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.binaryfile;

import java.io.OutputStream;
import java.util.List;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.file.OzgFileMapper;
import de.ozgcloud.common.binaryfile.FileId;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service(ArchiveManagerConfiguration.BINARY_FILE_REMOTE_SERVICE_NAME)
@RequiredArgsConstructor
class BinaryFileRemoteService {

	@GrpcClient(ArchiveManagerConfiguration.GRPC_VORGANG_MANAGER_CLIENT_NAME)
	private final BinaryFileServiceBlockingStub grpcService;

	@Qualifier(ArchiveManagerConfiguration.OZGCLOUD_FILE_SERVICE_NAME)
	private final OzgCloudFileService ozgCloudFileService;

	private final OzgFileMapper ozgFileMapper;

	@Qualifier(ArchiveManagerConfiguration.CALL_CONTEXT_PROVIDER_NAME)
	private final OzgCloudCallContextProvider contextProvider;

	@Qualifier(ArchiveManagerConfiguration.FILE_ID_MAPPER_NAME)
	private final FileIdMapper fileIdMapper;

	public Stream<OzgFile> getFiles(List<FileId> fileIds) {
		var response = getGrpcServiceWithInterceptor().findBinaryFilesMetaData(buildBinaryFilesRequest(fileIds));

		return mapGrpcFindFilesRespone(response);
	}

	GrpcBinaryFilesRequest buildBinaryFilesRequest(List<FileId> fileIds) {
		return GrpcBinaryFilesRequest.newBuilder()
				.addAllFileId(mapFileIds(fileIds))
				.build();
	}

	private List<String> mapFileIds(List<FileId> fileIds) {
		return fileIds.stream().map(FileId::toString).toList();
	}

	Stream<OzgFile> mapGrpcFindFilesRespone(GrpcFindFilesResponse response) {
		return response.getFileList().stream().map(ozgFileMapper::fromGrpc);
	}

	public OzgFile getFile(FileId fileId) {
		return ozgFileMapper.fromOzgCloudFile(ozgCloudFileService.getFile(fileIdMapper.toOzgCloudFileId(fileId)));
	}

	public void writeFileContent(FileId fileId, OutputStream outputStream) {
		ozgCloudFileService.writeFileDataToStream(fileIdMapper.toOzgCloudFileId(fileId), outputStream);
	}

	BinaryFileServiceBlockingStub getGrpcServiceWithInterceptor() {
		return grpcService.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
