/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.export;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.bescheid.ExportBescheidService;
import de.ozgcloud.archive.common.ExportFilenameGenerator;
import de.ozgcloud.archive.file.ExportFileService;
import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.historie.ExportHistorieService;
import de.ozgcloud.archive.kommentar.ExportKommentarService;
import de.ozgcloud.archive.postfach.ExportNachrichtService;
import de.ozgcloud.archive.vorgang.Eingang;
import de.ozgcloud.archive.vorgang.EingangHeader;
import de.ozgcloud.archive.vorgang.VorgangService;
import de.ozgcloud.archive.vorgang.VorgangWithEingang;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.xoev.xdomea.AbgabeAbgabe0401;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportService {

	static final String EXPORT_FILENAME_SUFFIX = "_Abgabe.Abgabe.0401.xml";
	private static final String EXPORT_FILENAME_TEMPLATE = "%s" + EXPORT_FILENAME_SUFFIX;
	private static final String EXPORT_XDOMEA_FILENAME_TEMPLATE = "%s_Abgabe.Abgabe.0401.xdomea";

	private final XdomeaXmlMarshaller xDomeaXmlMarshaller;

	private final ExportFileService exportFileService;
	@Qualifier(ArchiveManagerConfiguration.VORGANG_SERVICE_NAME)
	private final VorgangService vorgangService;
	private final ExportHistorieService exportHistorieService;
	@Qualifier(ArchiveManagerConfiguration.KOMMENTAR_SERVICE_NAME)
	private final ExportKommentarService exportKommentarService;
	private final ExportNachrichtService exportNachrichtService;
	private final ExportBescheidService exportBescheidService;

	public String buildXdomeaFileName(String fileNameId) {
		return String.format(EXPORT_XDOMEA_FILENAME_TEMPLATE, fileNameId);
	}

	public void writeXdomeaFileContent(String vorgangId, String filenameId, OutputStream outputStream) {
		var exportData = collectExportData(vorgangId, filenameId);
		writeZipFile(exportData, outputStream);
	}

	ExportData collectExportData(String vorgangId, String filenameId) {
		var vorgang = vorgangService.getVorgang(vorgangId);
		var kommentarsData = exportKommentarService.createExportData(vorgang);
		var postfachMailData = exportNachrichtService.createExportData(vorgang);
		var bescheidData = exportBescheidService.createExportData(vorgang);
		var representations = exportFileService.getRepresentations(vorgang).toList();
		var attachments = exportFileService.getAttachments(vorgang).toList();
		var formEngineName = getFormEngineName(vorgang);
		var abgabe = XdomeaNachrichtBuilder.builder()
				.withKopf(vorgangService.createKopf(vorgang))
				.withVorgang(vorgangService.createVorgangType(vorgang))
				.withAktenzeichen(vorgangService.createAkteType(vorgang))
				.withRepresentations(exportFileService.createDokumentTypes(representations, formEngineName).toList())
				.withAttachments(exportFileService.createDokumentTypes(attachments, formEngineName).toList())
				.withHistorie(exportHistorieService.createHistorienProtokollInformationTypes(vorgang).toList())
				.withKommentare(kommentarsData.getDokumentTypes())
				.withPostfachMails(postfachMailData.getDokumentTypes())
				.withBescheids(bescheidData.getDokumentTypes())
				.build();
		var exportFiles = Stream
				.of(representations.stream(), attachments.stream(),
						kommentarsData.getAttachments().stream(),
						postfachMailData.getAttachments().stream(),
						bescheidData.getFiles().stream())
				.flatMap(Function.identity())
				.collect(Collectors.toSet());
		return ExportData.builder().abgabe(abgabe).exportFilename(buildXmlFilename(filenameId)).exportFiles(exportFiles).build();
	}

	String getFormEngineName(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getEingang())
				.map(Eingang::getHeader)
				.map(EingangHeader::getFormEngineName).orElse(StringUtils.EMPTY);
	}

	String buildXmlFilename(String filenameId) {
		return String.format(EXPORT_FILENAME_TEMPLATE, filenameId);
	}

	String createXmlContent(AbgabeAbgabe0401 abgabe) {
		return xDomeaXmlMarshaller.marshal(abgabe);
	}

	void writeZipFile(ExportData exportData, OutputStream outputStream) {
		try (var zipOutputStream = new ZipOutputStream(outputStream)) {
			putZipEntry(exportData.getExportFilename(), createXmlContent(exportData.getAbgabe()), zipOutputStream);
			putFilesIntoZip(exportData.getExportFiles(), zipOutputStream);
		} catch (Exception e) {
			throw new TechnicalException("Error creating xdomea zip file", e);
		}
	}

	void putZipEntry(String fileName, String fileData, ZipOutputStream zipOutputStream) throws IOException {
		var entry = new ZipEntry(fileName);
		zipOutputStream.putNextEntry(entry);
		zipOutputStream.write(fileData.getBytes(StandardCharsets.UTF_8));
		zipOutputStream.closeEntry();
	}

	private void putFilesIntoZip(Set<OzgFile> ozgFiles, ZipOutputStream zipOutputStream) {
		ozgFiles.forEach(ozgFile -> putOzgFileIntoZip(ozgFile, zipOutputStream));
	}

	void putOzgFileIntoZip(OzgFile ozgFile, ZipOutputStream zipOutputStream) {
		try {
			var entry = new ZipEntry(ExportFilenameGenerator.generateExportFilename(ozgFile));
			zipOutputStream.putNextEntry(entry);
			exportFileService.writeOzgFile(ozgFile.getId(), zipOutputStream);
			zipOutputStream.closeEntry();
		} catch (IOException e) {
			throw new TechnicalException("Cannot add file to ZIP.", e);
		}
	}
}
