/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.kommentar;

import java.util.Collections;
import java.util.List;

import de.ozgcloud.archive.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.archive.common.DateConverter;
import de.ozgcloud.archive.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.archive.file.OzgFile;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class DokumentTypeBuilder {

	static final String TYP = "Notiz";
	static final String AKTION = "CREATE_KOMMENTAR";

	private Kommentar kommentar;
	private List<OzgFile> kommentarAttachments = Collections.emptyList();
	private String authorFullName;
	private String organisationseinheitenID;

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withKommentar(Kommentar kommentar) {
		this.kommentar = kommentar;
		return this;
	}

	public DokumentTypeBuilder withKommentarAttachments(List<OzgFile> kommentarAttachments) {
		this.kommentarAttachments = kommentarAttachments;
		return this;
	}

	public DokumentTypeBuilder withAuthorFullName(String authorFullName) {
		this.authorFullName = authorFullName;
		return this;
	}

	public DokumentTypeBuilder withOrganisationseinheitenID(String organisationseinheitenID) {
		this.organisationseinheitenID = organisationseinheitenID;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(IdentifikationObjektTypeBuilder.builder()
				.withObjectID(kommentar.getId())
				.build());
		dokumentType.setTyp(TYP);
		dokumentType.getHistorienProtokollInformation().add(createHistorie());
		kommentarAttachments.stream().map(this::createAnlage).forEach(dokumentType.getAnlage()::add);
		return dokumentType;
	}

	AnlageDokumentType createAnlage(OzgFile ozgFile) {
		return AnlageDokumentTypeBuilder.builder().withOzgFile(ozgFile).build();
	}

	HistorienProtokollInformationType createHistorie() {
		var historienProtokollInformationType = new HistorienProtokollInformationType();
		historienProtokollInformationType.setMetadatumName(kommentar.getText());
		historienProtokollInformationType.setAkteur(createAkteur());
		historienProtokollInformationType.setDatumUhrzeit(DateConverter.toXmlGregorianCalendar(kommentar.getCreatedAt()));
		historienProtokollInformationType.setAktion(AKTION);
		return historienProtokollInformationType;
	}

	String createAkteur() {
		return this.authorFullName + "; " + this.organisationseinheitenID;
	}

}
