/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.callcontext;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.archive.common.user.OzgCloudUserIdMapper;
import de.ozgcloud.common.grpc.GrpcUtil;
import io.grpc.Metadata;

class CallContextUserMapperTest {

	@InjectMocks
	private final CallContextMapper mapper = Mappers.getMapper(CallContextMapper.class);

	@Mock
	private OzgCloudUserIdMapper ozgCloudUserIdMapper;

	@Nested
	class TestToOzgCloudCallContext {

		@BeforeEach
		void setUpCallToUserIdMapper() {
			when(ozgCloudUserIdMapper.fromString(any(Optional.class))).thenReturn(CallContextTestFactory.USER_ID);
		}

		@Test
		void shouldMap() {
			var mappedContext = CallContextTestFactory.createBuilder().clientName(CallContextUser.ARCHIVE_MANAGER_CLIENT_NAME).build();

			var result = mapper.toOzgCloudCallContext(CallContextUserTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(mappedContext);
		}
	}

	@Nested
	class TestToCallContextUser {

		@Mock
		private Metadata headers;

		@Test
		void shouldMapRequestId() {
			var result = toCallContextUser();

			assertThat(result.getRequestId()).isSameAs(CallContextUserTestFactory.REQUEST_ID);
		}

		@Test
		void shouldMapClientName() {
			try (var grpcUtilMock = mockStatic(GrpcUtil.class)) {
				grpcUtilMock.when(() -> GrpcUtil.getClientName(headers)).thenReturn(Optional.of(CallContextUserTestFactory.CLIENT_NAME));

				var result = toCallContextUser();

				grpcUtilMock.verify(() -> GrpcUtil.getClientName(headers));
				assertThat(result.getClientName()).isSameAs(CallContextUserTestFactory.CLIENT_NAME);
			}
		}

		@Test
		void shouldMapUserId() {
			try (var grpcUtilMock = mockStatic(GrpcUtil.class)) {
				grpcUtilMock.when(() -> GrpcUtil.getUserId(headers)).thenReturn(Optional.of(CallContextUserTestFactory.USER_ID));

				var result = toCallContextUser();

				grpcUtilMock.verify(() -> GrpcUtil.getUserId(headers));
				assertThat(result.getUserId()).contains(CallContextUserTestFactory.USER_ID);

			}
		}

		@Test
		void shouldMapUserName() {
			try (var grpcUtilMock = mockStatic(GrpcUtil.class)) {
				grpcUtilMock.when(() -> GrpcUtil.getUserName(headers)).thenReturn(Optional.of(CallContextUserTestFactory.USER_NAME));

				var result = toCallContextUser();

				grpcUtilMock.verify(() -> GrpcUtil.getUserName(headers));
				assertThat(result.getUserName()).contains(CallContextUserTestFactory.USER_NAME);
			}
		}

		@Test
		void shouldMapAccessLimited() {
			try (var grpcUtilMock = mockStatic(GrpcUtil.class)) {
				grpcUtilMock.when(() -> GrpcUtil.getAccessLimited(headers)).thenReturn(Optional.of(CallContextUserTestFactory.ACCESS_LIMITED));
				grpcUtilMock.when(() -> GrpcUtil.getAccessLimitedOrgaIds(headers))
						.thenReturn(List.of(CallContextUserTestFactory.ORGANISATIONS_EINHEITEN_ID));

				var result = toCallContextUser();

				grpcUtilMock.verify(() -> GrpcUtil.getAccessLimited(headers));
				grpcUtilMock.verify(() -> GrpcUtil.getAccessLimitedOrgaIds(headers));
				assertThat(result.isAccessLimited()).isEqualTo(CallContextUserTestFactory.ACCESS_LIMITED);
				assertThat(result.getAccessLimitedToOrgaIds()).containsExactly(CallContextUserTestFactory.ORGANISATIONS_EINHEITEN_ID);
			}
		}

		@Test
		void shouldMapNoAccessLimited() {
			try (var grpcUtilMock = mockStatic(GrpcUtil.class)) {
				grpcUtilMock.when(() -> GrpcUtil.getAccessLimited(any())).thenReturn(Optional.empty());
				grpcUtilMock.when(() -> GrpcUtil.getAccessLimitedOrgaIds(any()))
						.thenReturn(List.of(CallContextTestFactory.ORGANISATIONS_EINHEITEN_ID));

				var result = toCallContextUser();

				grpcUtilMock.verify(() -> GrpcUtil.getAccessLimited(headers));
				assertThat(result.isAccessLimited()).isFalse();
				assertThat(result.getAccessLimitedToOrgaIds()).isEmpty();
			}
		}

		private CallContextUser toCallContextUser() {
			return mapper.toCallContextUser(headers, CallContextUserTestFactory.REQUEST_ID);
		}
	}
}
