/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Optional;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedConstruction;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.common.callcontext.CallContextProvider;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcCommandsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcExistsPendingCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcExistsPendingCommandsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcFindCommandsRequest;

class CommandRemoteServiceTest {

	@Spy
	@InjectMocks
	private CommandRemoteService service;
	@Mock
	private CommandServiceBlockingStub grpcService;
	@Mock
	private CommandMapper mapper;
	@Mock
	private CallContextProvider contextProvider;

	@Nested
	class TestFindCommands {

		private static final String COMMAND_STATUS_FINISHED = "FINISHED";
		private static final String REDIRECT_VORGANG_ORDER = "REDIRECT_VORGANG";
		private final GrpcCommand grpcCommand = GrpcCommand.newBuilder().build();
		private final GrpcCommandsResponse response = GrpcCommandsResponse.newBuilder().addAllCommand(Collections.singleton(grpcCommand)).build();

		@Mock
		private CommandServiceBlockingStub grpcServiceWithInterceptor;

		@Captor
		private ArgumentCaptor<GrpcFindCommandsRequest> requestCaptor;

		@BeforeEach
		void init() {
			doReturn(grpcServiceWithInterceptor).when(service).getGrpcServiceWithInterceptor();
			when(grpcServiceWithInterceptor.findCommands(any())).thenReturn(response);
		}

		@Test
		void shouldGetGrpcServiceWithInterceptor() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.empty(), Optional.empty());

			verify(service).getGrpcServiceWithInterceptor();
		}

		@Test
		void shouldCallGrpcService() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.empty(), Optional.empty());

			verify(grpcServiceWithInterceptor).findCommands(notNull());
		}

		@Test
		void shouldRequestWithVorgangId() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.empty(), Optional.empty());

			verify(grpcServiceWithInterceptor).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldRequestWithStatus() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.of(COMMAND_STATUS_FINISHED), Optional.empty());

			verify(grpcServiceWithInterceptor).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getStatusList()).contains(COMMAND_STATUS_FINISHED);
		}

		@Test
		void shouldRequestWithOrder() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.empty(), Optional.of(REDIRECT_VORGANG_ORDER));

			verify(grpcServiceWithInterceptor).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getOrderString()).isEqualTo(REDIRECT_VORGANG_ORDER);
		}

		@Test
		void shouldCallMapper() {
			service.findCommands(VorgangWithEingangTestFactory.ID, Optional.empty(), Optional.empty()).toList();

			verify(mapper).fromGrpc(grpcCommand);
		}
	}

	@Nested
	class TestGetGrpcServiceWithInterceptor {

		private MockedConstruction<OzgCloudCallContextAttachingInterceptor> mockedConstructionInterceptor;
		private OzgCloudCallContextProvider argumentContextProvider;
		@Mock
		private OzgCloudCallContextAttachingInterceptor interceptor;

		@BeforeEach
		void setUpMocks() {
			mockedConstructionInterceptor = mockConstruction(OzgCloudCallContextAttachingInterceptor.class, (mock, context) -> {
				argumentContextProvider = (OzgCloudCallContextProvider) context.arguments().get(0);
				interceptor = mock;
			});
		}

		@AfterEach
		void cleanUp() {
			mockedConstructionInterceptor.close();
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptor() {
			callService();

			assertThat(mockedConstructionInterceptor.constructed()).hasSize(1);
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptorWithContextProvider() {
			callService();

			assertThat(argumentContextProvider).isEqualTo(contextProvider);
		}

		@Test
		void shouldSetInterceptor() {
			callService();

			verify(grpcService).withInterceptors(interceptor);
		}

		@Test
		void shouldReturnServiceStub() {
			when(grpcService.withInterceptors(any(OzgCloudCallContextAttachingInterceptor.class))).thenReturn(grpcService);

			var returnedServiceStub = callService();

			assertThat(returnedServiceStub).isEqualTo(grpcService);
		}

		private CommandServiceBlockingStub callService() {
			return service.getGrpcServiceWithInterceptor();
		}
	}

	@Nested
	class TestHasPendingCommands {

		private final GrpcExistsPendingCommandsRequest request = GrpcExistsPendingCommandsRequestTestFactory.create();

		@Mock
		private CommandServiceBlockingStub serviceStubWithInterceptor;

		@BeforeEach
		void mock() {
			doReturn(serviceStubWithInterceptor).when(service).getGrpcServiceWithInterceptor();
			doReturn(request).when(service).buildHasPendingCommandRequest(VorgangWithEingangTestFactory.ID);
			var response = GrpcExistsPendingCommandsResponse.newBuilder().setExistsPendingCommands(true).build();
			when(serviceStubWithInterceptor.existsPendingCommands(any())).thenReturn(response);
		}

		@Test
		void shouldCallBuildHasPendingCommandRequest() {
			hasPendingCommands();

			verify(service).buildHasPendingCommandRequest(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldCallGetCommandServiceStub() {
			hasPendingCommands();

			verify(service).getGrpcServiceWithInterceptor();
		}

		@Test
		void shouldCallServiceStubWithInterceptor() {
			hasPendingCommands();

			verify(serviceStubWithInterceptor).existsPendingCommands(request);
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldReturnIfCommandsArePending(boolean hasPendingCommands) {
			var response = GrpcExistsPendingCommandsResponse.newBuilder().setExistsPendingCommands(hasPendingCommands).build();
			when(serviceStubWithInterceptor.existsPendingCommands(any())).thenReturn(response);

			var result = hasPendingCommands();

			assertThat(result).isEqualTo(hasPendingCommands);
		}

		private boolean hasPendingCommands() {
			return service.hasPendingCommands(VorgangWithEingangTestFactory.ID);
		}
	}

	@Nested
	class TestBuildHasPendingCommandRequest {

		@Test
		void shouldReturnRequest() {
			var request = service.buildHasPendingCommandRequest(VorgangWithEingangTestFactory.ID);

			assertThat(request).isEqualTo(GrpcExistsPendingCommandsRequestTestFactory.create());
		}
	}
}
