/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.OutputStream;
import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.springframework.http.MediaType;

import de.ozgcloud.archive.common.binaryfile.BinaryFileService;
import de.ozgcloud.archive.export.DokumentTypeTestFactory;
import de.ozgcloud.archive.vorgang.EingangHeaderTestFactory;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.common.binaryfile.FileId;
import de.xoev.xdomea.DokumentType;

class ExportFileServiceTest {

	@InjectMocks
	private ExportFileService service;

	@Mock
	private OzgFileRemoteService ozgFileRemoteService;

	@Mock
	private BinaryFileService binaryFileService;

	@Nested
	class TestGetRepresentations {

		private final OzgFile representationPdfFile = OzgFileTestFactory.createBuilder().contentType(MediaType.APPLICATION_PDF_VALUE).build();
		private final OzgFile representationXmlFile = OzgFileTestFactory.createBuilder().contentType(MediaType.TEXT_XML_VALUE).build();

		@BeforeEach
		void init() {
			when(ozgFileRemoteService.getRepresentations(VorgangWithEingangTestFactory.ID)).thenReturn(
					Stream.of(representationPdfFile, representationXmlFile));
		}

		@Test
		void shouldLoadRepresentations() {
			service.getRepresentations(VorgangWithEingangTestFactory.create()).toList();

			verify(ozgFileRemoteService).getRepresentations(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldReturnAllFiles() {
			var representations = service.getRepresentations(VorgangWithEingangTestFactory.create());

			assertThat(representations).hasSize(2).containsExactlyInAnyOrder(representationPdfFile, representationXmlFile);
		}

	}

	@Nested
	class TestGetAttachments {

		private final OzgFile attachmentPdfFile = OzgFileTestFactory.createBuilder().contentType(MediaType.APPLICATION_PDF_VALUE).build();
		private final OzgFile attachmentXmlFile = OzgFileTestFactory.createBuilder().contentType(MediaType.TEXT_XML_VALUE).build();

		@BeforeEach
		void init() {
			when(ozgFileRemoteService.getAttachments(VorgangWithEingangTestFactory.ID))
					.thenReturn(Stream.of(attachmentXmlFile, attachmentPdfFile));
		}

		@Test
		void shouldLoadAttachments() {
			service.getAttachments(VorgangWithEingangTestFactory.create()).toList();

			verify(ozgFileRemoteService).getAttachments(VorgangWithEingangTestFactory.ID);
		}

		@Test
		void shouldReturnAllFiles() {
			var ozgFiles = service.getAttachments(VorgangWithEingangTestFactory.create());

			assertThat(ozgFiles).containsExactlyInAnyOrder(attachmentPdfFile, attachmentXmlFile);
		}

	}

	@Nested
	class TestWriteOzgFile {

		private final FileId fileId = OzgFileTestFactory.ID;

		@Mock
		private OutputStream outputStream;

		@Test
		void shouldCallBinaryFileService() {
			service.writeOzgFile(fileId, outputStream);

			verify(binaryFileService).writeFileContent(fileId, outputStream);
		}
	}

	@Nested
	class TestCreateDokumentType {

		private MockedStatic<DokumentTypeBuilder> dokumentTypeBuilderMockedStatic;

		@Mock
		private DokumentTypeBuilder dokumentTypeBuilder;

		private final OzgFile ozgFile = OzgFileTestFactory.create();
		private final DokumentType dokumentType = DokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			dokumentTypeBuilderMockedStatic = mockStatic(DokumentTypeBuilder.class);
			dokumentTypeBuilderMockedStatic.when(DokumentTypeBuilder::builder).thenReturn(dokumentTypeBuilder);

			when(dokumentTypeBuilder.withOzgFile(ozgFile)).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.withOrdinalNumber(1L)).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.withFormEngineName(EingangHeaderTestFactory.FORM_ENGINE_NAME)).thenReturn(dokumentTypeBuilder);
			when(dokumentTypeBuilder.build()).thenReturn(dokumentType);
		}

		@AfterEach
		void tearDown() {
			dokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldBuild() {
			service.createDokumentTypes(List.of(ozgFile), EingangHeaderTestFactory.FORM_ENGINE_NAME).toList();

			verify(dokumentTypeBuilder).build();
		}

		@Test
		void shouldSetOzgFile() {
			service.createDokumentTypes(List.of(ozgFile), EingangHeaderTestFactory.FORM_ENGINE_NAME).toList();

			verify(dokumentTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldSetOrdinalNumber() {
			service.createDokumentTypes(List.of(ozgFile), EingangHeaderTestFactory.FORM_ENGINE_NAME).toList();

			verify(dokumentTypeBuilder).withOrdinalNumber(1L);
		}

		@Test
		void shouldSetFormEngineName() {
			service.createDokumentTypes(List.of(ozgFile), EingangHeaderTestFactory.FORM_ENGINE_NAME).toList();

			verify(dokumentTypeBuilder).withFormEngineName(EingangHeaderTestFactory.FORM_ENGINE_NAME);
		}

		@Test
		void shouldReturnDokumentTypes() {
			var dokumentTypes = service.createDokumentTypes(List.of(ozgFile), EingangHeaderTestFactory.FORM_ENGINE_NAME);

			assertThat(dokumentTypes).containsExactly(dokumentType);
		}
	}

}