/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.historie;

import static de.ozgcloud.archive.historie.AktenzeichenChangeHistoryBuilder.*;
import static de.ozgcloud.archive.historie.ChangeHistoryBuilderTest.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandTestFactory;
import de.ozgcloud.archive.common.command.CommandWithPrevious;
import de.ozgcloud.archive.common.command.CommandWithPreviousTestFactory;

public class AktenzeichenChangeHistoryBuilderTest {

	private static final String ORGANISATIONSEINHEITEN_ID = UUID.randomUUID().toString();

	private final ArchiveManagerCommand previousCommand = commandFinishedAt(LocalDateTime.of(2023, 5, 1, 12, 0));
	private final ArchiveManagerCommand command = commandFinishedAt(LocalDateTime.of(2023, 6, 1, 12, 0));
	private final List<ArchiveManagerCommand> commands = List.of(previousCommand, command);

	@Spy
	private AktenzeichenChangeHistoryBuilder builder = AktenzeichenChangeHistoryBuilder.builder()
			.withCommands(commands)
			.withOrganisationseinheitenID(ORGANISATIONSEINHEITEN_ID);

	@Nested
	class TestIsRelevant {

		@Test
		void shouldReturnTrue() {
			var command = CommandTestFactory.createBuilder()
					.order(SET_AKTENZEICHEN_ORDER)
					.build();

			var result = builder.isRelevant(command);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var command = CommandTestFactory.createBuilder()
					.order(LoremIpsum.getInstance().getWords(1))
					.build();

			var result = builder.isRelevant(command);

			assertThat(result).isFalse();
		}
	}

	@Nested
	class TestToCommandWithChangeValues {

		private static final String EXPECTED_AKTENZEICHEN_BEFORE_CHANGE = LoremIpsum.getInstance().getWords(2);
		private static final String EXPECTED_AKTENZEICHEN_AFTER_CHANGE = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@BeforeEach
		void init() {
			when(builder.getAktenzeichenBeforeChange(commandWithPrevious)).thenReturn(EXPECTED_AKTENZEICHEN_BEFORE_CHANGE);
			when(builder.getAktenzeichenAfterChange(commandWithPrevious)).thenReturn(EXPECTED_AKTENZEICHEN_AFTER_CHANGE);
		}

		@Test
		void shouldSetCommand() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.command()).isEqualTo(command);
		}

		@Test
		void shouldSetValueBeforeChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueBeforeChange()).isEqualTo(EXPECTED_AKTENZEICHEN_BEFORE_CHANGE);
		}

		@Test
		void shouldSetValueAfterChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueAfterChange()).isEqualTo(EXPECTED_AKTENZEICHEN_AFTER_CHANGE);
		}

		private ChangeHistoryBuilder.CommandWithChangeValues callBuilder() {
			return builder.toCommandWithChangeValues(commandWithPrevious);
		}
	}

	@Nested
	class TestGetAktenzeichenBeforeChange {

		private final CommandWithPrevious commandWithoutPrevious = CommandWithPreviousTestFactory.createBuilder().command(previousCommand)
				.previous(null).build();
		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@Test
		void shouldReturnEmptyIfPreviousIsNull() {
			var aktenzeichen = builder.getAktenzeichenBeforeChange(commandWithoutPrevious);

			assertThat(aktenzeichen).isEmpty();
		}

		@Test
		void shouldGetAktenzeichenFromPreviousCommand() {
			var expectedValue = LoremIpsum.getInstance().getWords(2);
			when(builder.getAktenzeichen(previousCommand)).thenReturn(expectedValue);

			var aktenzeichen = builder.getAktenzeichenBeforeChange(commandWithPrevious);

			assertThat(aktenzeichen).isEqualTo(expectedValue);
		}
	}

	@Nested
	class TestGetAktenzeichenAfterChange {

		private static final String EXPECTED_AKTENZEICHEN = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@BeforeEach
		void init() {
			doReturn(EXPECTED_AKTENZEICHEN).when(builder).getAktenzeichen(command);
		}

		@Test
		void shouldGetAktenzeichenFromCommand() {
			builder.getAktenzeichenAfterChange(commandWithPrevious);

			verify(builder).getAktenzeichen(command);
		}

		@Test
		void shouldReturnAktenzeichen() {
			var result = builder.getAktenzeichenAfterChange(commandWithPrevious);

			assertThat(result).isEqualTo(EXPECTED_AKTENZEICHEN);
		}
	}

	@Nested
	class TestGetAktenzeichen {

		private static final String AKTENZEICHEN = LoremIpsum.getInstance().getWords(1);

		@BeforeEach
		void init() {
			when(builder.getValueFromCommandBodyObject(BODY_PROPERTY_AKTENZEICHEN, previousCommand)).thenReturn(AKTENZEICHEN);
		}

		@Test
		void shouldGetValueFromCommandBody() {
			builder.getAktenzeichen(previousCommand);

			verify(builder).getValueFromCommandBodyObject(BODY_PROPERTY_AKTENZEICHEN, previousCommand);
		}

		@Test
		void shouldReturnValueFromCommandBody() {
			var value = builder.getAktenzeichen(previousCommand);

			assertThat(value).isEqualTo(AKTENZEICHEN);
		}
	}
}
