/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collection;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.archive.common.TimeMapper;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;

class KommentarMapperTest {

	@Spy
	@InjectMocks
	private KommentarMapper mapper = Mappers.getMapper(KommentarMapper.class);
	@Mock
	private GrpcObjectMapper grpcObjectMapper;
	@Mock
	private TimeMapper timeMapper;
	@Mock
	private AttachmentMapper attachmentMapper;

	@Nested
	class TestFromItem {

		@BeforeEach
		void init() {
			doReturn(KommentarTestFactory.KOMMENTAR).when(mapper).mapItemToKommentar(GrpcKommentarTestFactory.ITEM);
		}

		@Test
		void shouldMapItemToKommentar() {
			callMapper();

			verify(mapper).mapItemToKommentar(GrpcKommentarTestFactory.ITEM);
		}

		@Test
		void shouldReturnKommentar() {
			var kommentar = callMapper();

			assertThat(kommentar).isEqualTo(KommentarTestFactory.KOMMENTAR);
		}

		private Kommentar callMapper() {
			return mapper.fromItem(GrpcKommentarTestFactory.create());
		}
	}

	@Nested
	class TestMapItemToKommentar {

		@BeforeEach
		void init() {
			doReturn(GrpcKommentarTestFactory.createItemAsMap()).when(mapper).mapItemToItemMap(GrpcKommentarTestFactory.ITEM);
			doReturn(KommentarTestFactory.KOMMENTAR).when(mapper).mapItemMapToKommentar(GrpcKommentarTestFactory.createItemAsMap());
		}

		@Test
		void shouldMapItemToItemMap() {
			callMapper();

			verify(mapper).mapItemToItemMap(GrpcKommentarTestFactory.ITEM);
		}

		@Test
		void shouldMapItemMapToKommentar() {
			callMapper();

			verify(mapper).mapItemMapToKommentar(GrpcKommentarTestFactory.createItemAsMap());
		}

		@Test
		void shouldReturnKommentar() {
			var kommentar = callMapper();

			assertThat(kommentar).isEqualTo(KommentarTestFactory.KOMMENTAR);
		}

		private Kommentar callMapper() {
			return mapper.mapItemToKommentar(GrpcKommentarTestFactory.ITEM);
		}
	}

	@Nested
	class TestMapItemToItemMap {

		@BeforeEach
		void init() {
			when(grpcObjectMapper.mapFromGrpc(GrpcKommentarTestFactory.ITEM)).thenReturn(GrpcKommentarTestFactory.createItemAsMap());
		}

		@Test
		void shouldMapFromGrpc() {
			callMapper();

			verify(grpcObjectMapper).mapFromGrpc(GrpcKommentarTestFactory.ITEM);
		}

		@Test
		void shouldReturnItemMap() {
			var itemMap = callMapper();

			assertThat(itemMap).isEqualTo(GrpcKommentarTestFactory.createItemAsMap());
		}

		Map<String, Object> callMapper() {
			return mapper.mapItemToItemMap(GrpcKommentarTestFactory.ITEM);
		}
	}

	@Nested
	class TestMapItemMapToKommentar {

		@BeforeEach
		void init() {
			doReturn(KommentarTestFactory.ID).when(mapper).mapObjectToString(KommentarTestFactory.ID);
			doReturn(KommentarTestFactory.TEXT).when(mapper).mapObjectToString(KommentarTestFactory.TEXT);
			doReturn(KommentarTestFactory.CREATED_BY).when(mapper).mapObjectToString(KommentarTestFactory.CREATED_BY);
			doReturn(KommentarTestFactory.CREATED_AT_STR).when(mapper).mapObjectToString(KommentarTestFactory.CREATED_AT_STR);

			when(timeMapper.parseString(KommentarTestFactory.CREATED_AT_STR)).thenReturn(KommentarTestFactory.CREATED_AT);
			when(attachmentMapper.mapAttachments(argThat(this::isCollectionOfAttachmentIds))).thenReturn(KommentarTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldMapIdToString() {
			callMapper();

			verify(mapper).mapObjectToString(KommentarTestFactory.ID);
		}

		@Test
		void shouldMapId() {
			var kommentar = callMapper();

			assertThat(kommentar.getId()).isEqualTo(KommentarTestFactory.ID);
		}

		@Test
		void shouldMapTextToString() {
			callMapper();

			verify(mapper).mapObjectToString(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldMapText() {
			var kommentar = callMapper();

			assertThat(kommentar.getText()).isEqualTo(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldMapCreatedByToString() {
			callMapper();

			verify(mapper).mapObjectToString(KommentarTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCreatedBy() {
			var kommentar = callMapper();

			assertThat(kommentar.getCreatedBy()).isEqualTo(KommentarTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCreatedAtToString() {
			callMapper();

			verify(mapper).mapObjectToString(KommentarTestFactory.CREATED_AT_STR);
		}

		@Test
		void shouldCallTimeMapper() {
			callMapper();

			verify(timeMapper).parseString(KommentarTestFactory.CREATED_AT_STR);
		}

		@Test
		void shouldMapCreatedAt() {
			var kommentar = callMapper();

			assertThat(kommentar.getCreatedAt()).isEqualTo(KommentarTestFactory.CREATED_AT);
		}

		@Test
		void shouldCallAttachmentMapper() {
			callMapper();

			verify(attachmentMapper).mapAttachments(argThat(this::isCollectionOfAttachmentIds));
		}

		@Test
		void shouldMapAttachments() {
			var kommentar = callMapper();

			assertThat(kommentar.getAttachments()).containsExactlyElementsOf(KommentarTestFactory.ATTACHMENTS);
		}

		@SuppressWarnings("unchecked")
		private boolean isCollectionOfAttachmentIds(Object obj) {
			return obj instanceof Collection && ((Collection<String>) obj).containsAll(GrpcKommentarTestFactory.ATTACHMENTS);
		}

		private Kommentar callMapper() {
			return mapper.mapItemMapToKommentar(GrpcKommentarTestFactory.createItemAsMap());
		}
	}

	@Nested
	class TestMapObjectToString {

		@Mock
		private Object object;

		@Test
		void shouldReturnNullIfObjectIsNull() {
			var result = mapper.mapObjectToString(null);

			assertThat(result).isNull();
		}

		@Test
		void shouldCallToString() {
			mapper.mapObjectToString(object);

			verify(mapper).mapObjectToString(object);
		}

		@Test
		void shouldReturnResultOfToString() {
			when(object.toString()).thenReturn(KommentarTestFactory.TEXT);

			var result = mapper.mapObjectToString(object);

			assertThat(result).isEqualTo(KommentarTestFactory.TEXT);
		}
	}
}
